from datetime import datetime
from decimal import Decimal

from pydantic import BaseModel, Field

from .assets import PerpetualAssetType


class MarketInfo(BaseModel):
    """This class `MarketInfo` defines attributes for market information including symbol, prices,
    funding rates, time, open interest, and 24-hour trading data.  It is used when fetching from the
    orderly API


    Attributes
    ----------
    symbol : [PerpetualAssetType](assets.md)
        This is the asset for the market info
    """

    symbol: PerpetualAssetType
    index_price: Decimal
    mark_price: Decimal
    sum_unitary_funding: Decimal
    est_funding_rate: Decimal
    last_funding_rate: Decimal
    next_funding_time: datetime
    open_interest: Decimal
    last_24h_open: Decimal = Field(alias="24h_open")
    last_24h_close: Decimal = Field(alias="24h_close")
    last_24h_high: Decimal = Field(alias="24h_high")
    last_24h_low: Decimal = Field(alias="24h_low")
    last_24h_volume: Decimal = Field(alias="24h_volume")
    last_24h_amount: Decimal = Field(alias="24h_amount")



from enum import StrEnum


class Interval(StrEnum):
    """Timeframe Intervals for trading view"""

    minute = "1"
    five_minute = "5"
    fifteen_minute = "15"
    thirty_minute = "30"
    hour = "60"
    four_hour = "4h"
    twelve_hour = "12h"
    day = "1D"



"""Introduce the core types for interacting with Orderly

This module creates Pydantic BaseModel's for all the core data structures
"""

from .assets import PerpetualAssetType
from .interval import Interval
from .market import MarketInfo
from .orders import Order, OrderResponse, OrderType
from .position import Position, PositionData
from .register import OrderlyRegistration
from .status import OrderStatus, Status


__all__ = [
    "Interval",
    "MarketInfo",
    "Order",
    "OrderlyRegistration",
    "OrderResponse",
    "OrderStatus",
    "OrderType",
    "PerpetualAssetType",
    "Position",
    "PositionData",
    "Status",
]



from datetime import datetime
from decimal import Decimal
from enum import StrEnum
from typing import Annotated, Literal

from pydantic import BaseModel, WrapValidator

from .assets import PerpetualAssetType


class OrderType(StrEnum):
    LIMIT = "LIMIT"
    MARKET = "MARKET"
    IOC = "IOC"  # It matches as much as possible at the order_price. If not fully executed, then remaining quantity will be cancelled.
    FOK = "FOK"  # if the order can be fully executed at the order_price then the order gets fully executed otherwise would be cancelled without any execution.
    POST_ONLY = "POST_ONLY"  # if the order will be executed with any maker trades at the time of placement, then it will be cancelled without any execution.
    ASK = "ASK"  # the order price is guranteed to be the best ask price of the orderbook at the time it gets accepted.
    BID = "BID"  # the order price is guranteed to be the best bid price of the orderbook at the time it gets accepted.


class OrderResponse(BaseModel):
    order_id: int
    client_order_id: int | None
    order_type: Literal["LIMIT", "MARKET"]
    order_price: Decimal | None
    order_quantity: Decimal | None
    order_amount: Decimal | None


class Order(BaseModel):
    symbol: Annotated[
        PerpetualAssetType, WrapValidator(PerpetualAssetType.to_perp_or_unk)
    ]
    status: Literal[
        "NEW",
        "CANCELLED",
        "PARTIAL_FILLED",
        "FILLED",
        "REJECTED",
        "INCOMPLETE",
        "COMPLETED",
    ]
    side: Literal["BUY", "SELL"]
    order_id: int
    user_id: int
    price: Decimal | None
    type: Literal["LIMIT", "MARKET"]
    quantity: Decimal
    amount: Decimal | None
    visible: Decimal
    executed: Decimal
    total_fee: Decimal
    fee_asset: str
    client_order_id: str | None
    created_time: datetime
    updated_time: datetime
    average_executed_price: Decimal | None
    reduce_only: bool | None
    order_tag: str | None
    realized_pnl: Decimal

    @property
    def usd_value(self):
        if self.average_executed_price and self.quantity:
            return self.average_executed_price * self.quantity
        return 0



from enum import Enum


class Status(str, Enum):
    PENDING = "PENDING"  # not supported currently
    RUNNING = "RUNNING"
    PAUSED = "PAUSED"  # not supported currently
    TERMINATED = "TERMINATED"


class OrderStatus(str, Enum):
    PENDING = "PENDING"
    CANCELLED = "CANCELLED"
    COMPLETED = "COMPLETED"



from enum import Enum
from typing import Any


class PerpetualAssetType(str, Enum):
    """An Enum of all supported Perpetual Asset Types"""

    BTC = "PERP_BTC_USDC"
    ETH = "PERP_ETH_USDC"
    WOO = "PERP_WOO_USDC"
    AVAX = "PERP_AVAX_USDC"
    MATIC = "PERP_MATIC_USDC"
    ARB = "PERP_ARB_USDC"
    INJ = "PERP_INJ_USDC"
    ORDI = "PERP_ORDI_USDC"
    SUI = "PERP_SUI_USDC"
    SOL = "PERP_SOL_USDC"
    TIA = "PERP_TIA_USDC"
    LINK = "PERP_LINK_USDC"
    OP = "PERP_OP_USDC"
    JUP = "PERP_JUP_USDC"
    STRK = "PERP_STRK_USDC"
    BLUR = "PERP_BLUR_USDC"
    WIF = "PERP_WIF_USDC"
    W = "PERP_W_USDC"  # WORMHOLE
    BNB = "PERP_BNB_USDC"
    WLD = "PERP_WLD_USDC"
    APT = "PERP_APT_USDC"
    XRP = "PERP_XRP_USDC"
    DOGE = "PERP_DOGE_USDC"
    ENA = "PERP_ENA_USDC"
    NEAR = "PERP_NEAR_USDC"
    FTM = "PERP_FTM_USDC"
    ETHFI = "PERP_ETHFI_USDC"
    OMNI = "PERP_OMNI_USDC"
    MERL = "PERP_MERL_USDC"
    BCH = "PERP_BCH_USDC"
    ONDO = "PERP_ONDO_USDC"
    REZ = "PERP_REZ_USDC"
    SEI = "PERP_SEI_USDC"
    FIL = "PERP_FIL_USDC"
    TON = "PERP_TON_USDC"
    BOME = "PERP_BOME_USDC"
    MEW = "PERP_MEW_USDC"
    BONK = "PERP_1000BONK_USDC"
    PEPE = "PERP_1000PEPE_USDC"
    NOT = "PERP_NOT_USDC"
    AR = "PERP_AR_USDC"
    BRETT = "PERP_BRETT_USDC"
    IO = "PERP_IO_USDC"
    ZRO = "PERP_ZRO_USDC"
    STG = "PERP_STG_USDC"
    BLAST = "PERP_BLAST_USDC"

    UNKNOWN = "UNKNOWN"

    @staticmethod
    def to_perp_or_unk(value, handler) -> "PerpetualAssetType":
        """
        The function `to_perp_or_unk` takes a value, tries to create a `PerpetualAssetType` object from it,
        and returns `PerpetualAssetType.UNKNOWN` if there is a `ValueError`.

        Examples:
            >>> PerpetualAssetType("PERP_ZRO_USDC")
            PerpetualAssetType.ZRO
            >>> PerpetualAssetType("????")
            PerpetualAssetType.UNKNOWN

        Args:
            value (Any): The `value` parameter in the `to_perp_or_unk` function is the input value that will be used to create a `PerpetualAssetType` object. The function will attempt to create a `PerpetualAssetType` object from this value. If the value is valid

        Returns:
            PerpetualAssetType: a `PerpetualAssetType` object
        """
        try:
            return PerpetualAssetType(value)
        except ValueError:
            return PerpetualAssetType.UNKNOWN



import os

from setuptools import setup

def get_version():
    return os.getenv("PACKAGE_VERSION", "0.0.0")


setup(
    name='emp-orderly-types',
    version=get_version(),
    python_requires='>=3.10, <3.13',
    install_requires=[
        'base58==2.1.1',
        'cryptography==42.0.5',
        'eth-typing==4.1.0',
        'eth-utils==4.1.0',
        'httpx==0.27.0',
        'pandas==2.2.2',
        'pydantic>=2.7.0',
        'python-dotenv==1.0.1',
        'requests==2.31.0'
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)



from datetime import datetime
from decimal import Decimal
from typing import Annotated

from pydantic import BaseModel, WrapValidator

from .assets import PerpetualAssetType


class Position(BaseModel):
    symbol: Annotated[
        PerpetualAssetType, WrapValidator(PerpetualAssetType.to_perp_or_unk)
    ]
    position_qty: Decimal
    cost_position: Decimal
    last_sum_unitary_funding: Decimal
    pending_long_qty: Decimal
    pending_short_qty: Decimal
    settle_price: Decimal
    average_open_price: Decimal
    unsettled_pnl: Decimal
    mark_price: Decimal
    est_liq_price: Decimal
    timestamp: datetime
    imr: Decimal
    mmr: Decimal
    IMR_withdraw_orders: Decimal
    MMR_with_orders: Decimal
    pnl_24_h: Decimal
    fee_24_h: Decimal


class PositionData(BaseModel):
    margin_ratio: Decimal
    initial_margin_ratio: Decimal
    maintenance_margin_ratio: Decimal
    open_margin_ratio: Decimal
    current_margin_ratio_with_orders: Decimal
    initial_margin_ratio_with_orders: Decimal
    maintenance_margin_ratio_with_orders: Decimal
    total_collateral_value: Decimal
    free_collateral: Decimal
    rows: list[Position]
    total_pnl_24_h: Decimal
